/*
 * Auxiliary functions
 * Author: Michael Mampaey
 *
 */

#include <vector>
#include <cmath>
#include <string.h>

using namespace std;



inline double h(int x, int sum) {
	double h = - x * log2(x) - (sum-x) * log2(sum-x);
	return h / (double)sum + log2((double)sum);
}


inline double h(const int *array, int len, int sum) {
	double h = 0.0;
	int *ar = (int*)array + len;
	while (ar-- != array)
		if (*ar)
			h -= *ar * log2(*ar);
	return h / (double)sum + log2((double)sum);
}


inline double sub_h(const int *array, unsigned len, int sum, int exclude) {
	double h = 0.0;
	//unsigned mask = (1 << len) -1;
	unsigned b = (1 << exclude);
	for (unsigned i=0; i<len; ++i) {
		if ((i&b)==0) {
			int f = array[i] + array[i+b];
			if (f)
				h -= f * log2(f);
		}
	}
	return h / (double)sum + log2((double)sum);
}


inline bool same(double x, double y) {
	static const double epsilon = 1e-8;
	return (x < y) ? (y-x < epsilon) : (x-y < epsilon);
}


inline bool inarray(const int *array, int len, int elt) {
	int *ar = (int*)array + len;
	while (ar-- != array)
		if (*ar==elt) return true;
	return false;
}


inline int *make_subset(const int *array, int len, int exclude) {
	int *subset = new int[len-1];
	memcpy(subset, array, exclude*sizeof(int));
	memcpy(subset+exclude, array+exclude+1, (len-exclude-1)*sizeof(int));
	return subset;
}


inline int *make_subset_into(const int *array, int len, int exclude, int* into) {
	memcpy(into, array, exclude*sizeof(int));
	memcpy(into+exclude, array+exclude+1, (len-exclude-1)*sizeof(int));
	return into;
}




template <class T> inline unsigned int index_of(const vector<T> &v, const T &k) {
	unsigned int index = find(v.begin(), v.end(), k) - v.begin();
	if (index==v.size())
		return -1;
	return index;
}
