/*
 * Main miner class
 * Author: Michael Mampaey
 * Date: 2009-06-13
 *
 */

#include <set>
#include <map>
#include "data.h"
#include "item.h"
#include "trie.h"

using namespace std;


class Miner
{
public:
	Miner();
	~Miner();

	void set_data(string file)      { infilename = file; }
	void set_max_entropy(double me) { maxentropy= me; }
	void set_min_mu(double mu)      { minmu= mu; }
	void set_pattern_type(int t)    { pattern_type = t; }
	void set_redundancy(int rd)     { redundancy_pruning = rd; }
	void set_minsup(double ms)      { minsup = ms; }
	void set_generators(bool g)     { generators = g; }
	void set_ndi(bool n)            { ndi = n; }
	void set_maxk(int k)            { maxk = k; }
	void set_output(string of)      { if (of=="-") to_std_out = true; else outfilename = of; }
	void set_header(bool h)         { header = h; }
	void set_verbose(bool verb)     { verbose = verb; }

	void mine();

	enum PATTERN_TYPE {RULES, SETS};
	enum REDUNDANCY {NONE, MINIMAL, LOSSY};

	static const string name;
	static const string version;
	static const string infostring;
	static const string author;

protected:
	typedef pair<int,double> pid;

	void initiate();
	void cleanup();

	set<Item> *read_data();
	pair<Item**,int> prune_attributes(set<Item> *root);
	void set_mine(Item **attributes, int alen, int *attrset, int depth, Node<pid>* rel_node, int *closure=0, int close_len=0);
	void rule_mine(int *full, int len, double entropy, int *lhs, int llen, int *rhs, int rlen, int q=0, unsigned *checker=0, unsigned indx=0);
	set<Item> * resort_attributes(set<Item> *root);

	inline int *qie(const int *attrset, int len) const;

	inline bool is_consequent_redundant(const int *full, int len, const int *rhs, int rlen, double lhsent, double mu) const;
	inline bool is_antecedent_redundant(const int *lhs, int llen, const int *rhs, int rlen, const int *full, int len, double rhsent, double mu) const;
	inline bool is_antecedent_redundant(const int *lhs, int llen, int rhs, const int *full, int len, double rhsent, double mu) const;

	inline void print_set (const int *aset, int len, const int *closure, int close_len, double entropy, int d=0) const;
	inline void print_rule(const int *lhs, int llen, const int *rhs, int rlen, double entropy, double mu) const;
	inline void print_rule(const int *lhs, int llen, int rhs, double entropy, double mu) const;

	inline int * get_partition(int size) const;
	inline unsigned * get_checker(int size) const;


	string    infilename;
	string    outfilename;
	Data      *data;
	FILE      *out;
	bool      to_std_out;
	unsigned  t_cnt;
	vector<string> attribute_names;

	double maxentropy;
	double minmu;
	double minsup;
	int    pattern_type;
	int    redundancy_pruning;
	bool   generators;
	bool   ndi;
	int    maxk;
	bool   header;
	bool   verbose;

	Trie<pid> *sup_ent_trie;
	mutable map<int,double> attr_ents;
	int       **partitions;
	unsigned  **checkers;
	int       *resorted;

	unsigned cand_cnt;
	unsigned set_cnt;
	unsigned rule_cnt;

	static const bool sort_output = false;

};
