/*
 * MuMiner
 * Author: Michael Mampaey
 * Date: 2009-06-13
 */

#include <iostream>
#include <stdlib.h>
#include <string>

#include "miner.h"
#include "aux.h"

using namespace std;



int main(int argc, char *argv[])
{
	enum OPTIONS { HELP = 0, SETS, RULES, REDUNDANCY, MINSUP, GENERATORS, NDI , MAXK, OUTPUT, VERBOSE, HEADER, OPT_CNT };
	vector<string> long_opts = vector<string>(OPT_CNT);
	long_opts[HELP]        = "--help";
	long_opts[SETS]        = "--sets";
	long_opts[RULES]       = "--rules";
	long_opts[REDUNDANCY]  = "--redundant";
	long_opts[MINSUP]      = "--prune";
	long_opts[GENERATORS]  = "--generators";
	long_opts[NDI]         = "--ndi";
	long_opts[MAXK]        = "--maxk";
	long_opts[OUTPUT]      = "--output";
	long_opts[HEADER]      = "--header";
	long_opts[VERBOSE]     = "--verbose";
	vector<string> short_opts = vector<string>(OPT_CNT);
	short_opts[HELP]       = "-h";
	short_opts[SETS]       = "-s";
	short_opts[RULES]      = "-r";
	short_opts[REDUNDANCY] = "-rd";
	short_opts[MINSUP]     = "-p";
	short_opts[NDI]        = "-ndi";
	short_opts[GENERATORS] = "-g";
	short_opts[MAXK]       = "-k";
	short_opts[OUTPUT]     = "-o";
	short_opts[HEADER]     = "-hd";
	short_opts[VERBOSE]    = "-v";

	if (argc < 4) {
		cerr << Miner::infostring << endl << "by " << Miner::author << endl;
		cerr << "Usage: " << argv[0] << " datafile maxent minmu [-s|-r] [-rd r] [-p p] [-g] [-ndi] [-k k] [-o o] [-hd] [-v]" << endl;
		if (argc==1) {
			cerr << "type --help for more info" << endl;
		}
		else {
			int option = -1;
			if (argc > 1 && (option = index_of(short_opts, string(argv[1]))) == -1)
				option = index_of(long_opts, string(argv[1]));
			if (option==HELP) {
				string spacer = "  ";
				cerr << spacer << "datafile"          << "\t\t\t" << "transactional input dataset in fimi format" << endl;
				cerr << spacer << "maxent"            << "\t\t\t" << "maximum entropy threshold, range [0,n]" << endl;
				cerr << spacer << "minmu "            << "\t\t\t" << "minimum dependence threshold, range [0,1]" << endl;
				cerr << spacer << "-s,\t--sets"       << "\t\t\t" << "only mine attribute sets" << endl;
				cerr << spacer << "-r,\t--rules"      << "\t\t\t" << "mine association rules (default)" << endl;
				cerr << spacer << "-rd,\t--redundant" << "\t\t"   << "level of redundancy pruning (0:none, 1:minimal default, 2:lossy)" << endl;
				cerr << spacer << "-p,\t--prune"      << "\t\t\t" << "symmetric attribute pruning frequency threshold, range [0,1]" << endl;
				cerr << spacer << "-g,\t--generators" << "\t\t"   << "only mine generators (in conjunction with -s)" << endl;
				cerr << spacer << "-ndi,\t--ndi"      << "\t\t\t" << "only mine non-derivable attribute sets or rules" << endl;
				cerr << spacer << "-k,\t--maxk"       << "\t\t\t" << "maximum attribute set size" << endl;
				cerr << spacer << "-o,\t--output"     << "\t\t"   << "output file (use '-' for stdout)" << endl;
				cerr << spacer << "-hd,\t--header"    << "\t\t"   << "first line of input file is header" << endl;
				cerr << spacer << "-v,\t--verbose"    << "\t\t"   << "enable verbose mode" << endl;
				cerr << spacer << "-h,\t--help"       << "\t\t\t" << "display this help" << endl;
			}
		}
		exit(1);
	}
	else {
		string datafile     = argv[1];
		double maxent       = atof(argv[2]);
		double minmu        = atof(argv[3]);
		int    pattern_type = Miner::RULES;
		int    redundancy   = Miner::MINIMAL;
		double minsup       = 0.0;
		bool   generators   = false;
		bool   ndi          = false;
		int    maxk         = -1;
		string outfile      = "";
		bool   header       = false;
		bool   verbose      = false;

		for (int arg=4; arg<argc; ++arg) {
			int option;
			if ( (option = index_of(short_opts, string(argv[arg]))) == -1 )
				option = index_of(long_opts, string(argv[arg]));

			switch (option) {
			case SETS:
				pattern_type = Miner::SETS;
				break;
			case RULES:
				pattern_type = Miner::RULES;
				break;
			case REDUNDANCY:
				redundancy = atoi(argv[arg+1]);
				++arg;
				break;
			case MINSUP:
				minsup = atof(argv[arg+1]);
				++arg;
				break;
			case GENERATORS:
				generators = true;
				break;
			case NDI:
				ndi = true;
				break;
			case MAXK:
				maxk = atoi(argv[arg+1]);
				++arg;
				break;
			case HEADER:
				header = true;
				break;
			case OUTPUT:
				outfile = argv[arg+1];
				++arg;
				break;
			case VERBOSE:
				verbose = true;
				break;
			default:
				cerr << "Error: unknown parameter " << argv[arg] << endl;
				exit(1);
			}
		}


		Miner miner;
		miner.set_data(datafile);
		miner.set_max_entropy(maxent);
		miner.set_min_mu(minmu);

		miner.set_pattern_type(pattern_type);
		miner.set_redundancy(redundancy);
		miner.set_minsup(minsup);

		miner.set_generators(generators);
		miner.set_ndi(ndi);
		miner.set_maxk(maxk);

		if (outfile.size())
			miner.set_output(outfile);
		miner.set_header(header);

		miner.set_verbose(verbose);

		miner.mine();

	}

	exit(0);

}
