/*
 * TidList & Item classes
 * Author: Michael Mampaey
 */

#include "tidlist.h"
#include <string.h>



TidList::TidList(unsigned s) : size(0), listsize(s) {
	list = new unsigned[listsize];
}


TidList::TidList(TidList &t) : size(t.size), listsize(t.size) {
	list = new unsigned[size];
	memcpy(list, t.list, sizeof(int)*size);
}


TidList::~TidList() {
	delete [] list;
}


inline void TidList::resize(unsigned newsize) {
	if (newsize==0)
		newsize = size;
	if (newsize==listsize)
		return;
	listsize = newsize;
	unsigned *newlist = new unsigned[listsize];
	memcpy(newlist, list, sizeof(int)*size);
	delete [] list;
	list = newlist;
	return;
}


void TidList::add(unsigned elt) {
	if (size==listsize)
		resize(listsize+blocksize);
	list[size++] = elt;
	return;
}


TidList *TidList::intersect(const TidList *t) const {
	TidList *res = new TidList((size < t->size) ? (size) : (t->size));
	unsigned *ix1 = list;
	unsigned *ix2 = t->list;
	unsigned *end1 = ix1 + size;
	unsigned *end2 = ix2 + t->size;
	unsigned *ins = res->list;
	while (ix1 < end1 && ix2 < end2) {
		if (*ix1==*ix2) {
			*ins++ = *ix1++;
			++ix2;
		}
		else if (*ix1 < *ix2) {
			++ix1;
		}
		else {
			++ix2;
		}
	}
	res->size = ins - res->list;
	//if (res->listsize > autoresize * res->size) res->resize();
	return res;
}

TidList *TidList::difference(const TidList *t) const {
	TidList *res = new TidList(size);
	unsigned *ix1 = list;
	unsigned *ix2 = t->list;
	unsigned *end1 = ix1 + size;
	unsigned *end2 = ix2 + t->size;
	unsigned *ins = res->list;
	while (ix1 < end1 && ix2 < end2) {
		if (*ix1==*ix2) {
			++ix1;
			++ix2;
		}
		else if (*ix1 < *ix2) {
			*ins = *ix1;
			++ins;
			++ix1;
		}
		else {
			++ix2;
		}
	}
	while (ix1 < end1) {
		*ins = *ix1;
		++ins;
		++ix1;
	}
	res->size = ins - res->list;
	//if (res->listsize > autoresize * res->size) res->resize();
	return res;
}
